"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

var _errors = require("./errors");

var _constants = require("./constants");

var _makePostgresClient = _interopRequireDefault(require("./make-postgres-client"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const executeStatement = async (pool, sql, distinctConnection) => {
  let reconnectionTimes = 0;
  let useDistinctConnection = distinctConnection;
  let distinctConnectionMade = false;

  while (true) {
    let connection;

    try {
      if (useDistinctConnection) {
        connection = (0, _makePostgresClient.default)(pool);
        await connection.connect();
        distinctConnectionMade = true;
      } else {
        connection = await pool.getConnectPromise();
      }
    } catch (error) {
      if (!useDistinctConnection) {
        pool.getConnectPromise = pool.createGetConnectPromise();
      }

      throw (0, _errors.makeConnectionError)(error);
    }

    try {
      const result = await connection.query(sql);

      if (result != null && Array.isArray(result.rows)) {
        return result.rows;
      }

      return [];
    } catch (error) {
      if ((0, _errors.isServiceBusyError)(error)) {
        throw new _eventstoreBase.ServiceBusyError(error.message);
      } else if ((0, _errors.isTimeoutError)(error)) {
        throw new _eventstoreBase.RequestTimeoutError(error.message);
      } else if ((0, _errors.isConnectionTerminatedError)(error)) {
        if (!useDistinctConnection) {
          pool.getConnectPromise = pool.createGetConnectPromise();
        }

        if (reconnectionTimes > _constants.MAX_RECONNECTIONS) {
          throw new _eventstoreBase.ServiceBusyError(error.message);
        }

        useDistinctConnection = true;
        reconnectionTimes++;
      } else if (error != null && error.message === 'Client was closed and is not queryable') {
        if (reconnectionTimes > _constants.MAX_RECONNECTIONS) {
          throw new _eventstoreBase.ServiceBusyError(error.message);
        }

        useDistinctConnection = true;
        reconnectionTimes++;
      } else {
        throw error;
      }
    } finally {
      if (distinctConnectionMade) {
        connection.end(err => {
          return;
        });
      }
    }
  }
};

var _default = executeStatement;
exports.default = _default;
//# sourceMappingURL=execute-statement.js.map